Package: spatstat.sparse
Version: 3.2-0
Date: 2026-05-21
Title: Sparse Three-Dimensional Arrays and Linear Algebra Utilities
Authors@R: c(person("Adrian", "Baddeley", 
                    role = c("aut", "cre", "cph"),
       	            email = "Adrian.Baddeley@curtin.edu.au",
		    comment = c(ORCID="0000-0001-9499-8382")),
	     person("Rolf", "Turner", 
                    role = c("aut", "cph"),
 	            email="rolfturner@posteo.net",
		    comment=c(ORCID="0000-0001-5521-5218")),
	    person("Ege",   "Rubak", 
                    role = c("aut", "cph"),
		    email = "rubak@math.aau.dk",
		    comment=c(ORCID="0000-0002-6675-533X")))
Maintainer: Adrian Baddeley <Adrian.Baddeley@curtin.edu.au>
Depends: R (>= 3.5.0), stats, utils, methods, Matrix, abind, tensor
Imports: spatstat.utils (>= 3.2-3)
Description: Defines sparse three-dimensional arrays
	     and supports standard operations on them.
	     The package also includes utility functions for
	     matrix calculations that are common in
	     statistics, such as quadratic forms.
License: GPL (>= 2)
URL: http://spatstat.org/
NeedsCompilation: yes
ByteCompile: true
BugReports: https://github.com/spatstat/spatstat.sparse/issues
Packaged: 2026-05-21 03:14:44 UTC; adrian
Author: Adrian Baddeley [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-9499-8382>),
  Rolf Turner [aut, cph] (ORCID: <https://orcid.org/0000-0001-5521-5218>),
  Ege Rubak [aut, cph] (ORCID: <https://orcid.org/0000-0002-6675-533X>)
Repository: CRAN
Date/Publication: 2026-05-21 05:10:02 UTC
